@@ -1,8 +1,10 @@
+mod start_plugin;
 mod wasm_bridge;
 use log::info;
 use std::{collections::HashMap, fs, path::PathBuf};
 use wasmer::Store;
 
+use crate::screen::ScreenInstruction;
 use crate::{pty::PtyInstruction, thread_bus::Bus, ClientId};
 
 use wasm_bridge::WasmBridge;
@@ -20,7 +22,14 @@ use zellij_utils::{
 
 #[derive(Clone, Debug)]
 pub enum PluginInstruction {
-    Load(RunPlugin, usize, ClientId, Size), // plugin metadata, tab_index, client_ids
+    Load(
+        Option<bool>,   // should float
+        Option<String>, // pane title
+        RunPlugin,
+        usize, // tab index
+        ClientId,
+        Size,
+    ),
     Update(Vec<(Option<u32>, Option<ClientId>, Event)>), // Focused plugin / broadcast, client_id, event data
     Unload(u32),                                         // plugin_id
     Resize(u32, usize, usize),                           // plugin_id, columns, rows
@@ -33,6 +42,7 @@ pub enum PluginInstruction {
         usize, // tab_index
         ClientId,
     ),
+    ApplyCachedEvents(u32), // u32 is the plugin id
     Exit,
 }
 
@@ -47,6 +57,7 @@ impl From<&PluginInstruction> for PluginContext {
             PluginInstruction::AddClient(_) => PluginContext::AddClient,
             PluginInstruction::RemoveClient(_) => PluginContext::RemoveClient,
             PluginInstruction::NewTab(..) => PluginContext::NewTab,
+            PluginInstruction::ApplyCachedEvents(..) => PluginContext::ApplyCachedEvents,
         }
     }
 }
@@ -69,8 +80,21 @@ pub(crate) fn plugin_thread_main(
         let (event, mut err_ctx) = bus.recv().expect("failed to receive event on channel");
         err_ctx.add_call(ContextType::Plugin((&event).into()));
         match event {
-            PluginInstruction::Load(run, tab_index, client_id, size) => {
-                wasm_bridge.load_plugin(&run, tab_index, size, client_id)?;
+            PluginInstruction::Load(should_float, pane_title, run, tab_index, client_id, size) => {
+                match wasm_bridge.load_plugin(&run, tab_index, size, client_id) {
+                    Ok(plugin_id) => {
+                        drop(bus.senders.send_to_screen(ScreenInstruction::AddPlugin(
+                            should_float,
+                            run,
+                            pane_title,
+                            tab_index,
+                            plugin_id,
+                        )));
+                    },
+                    Err(e) => {
+                        log::error!("Failed to load plugin: {e}");
+                    },
+                }
             },
             PluginInstruction::Update(updates) => {
                 wasm_bridge.update_plugins(updates)?;
@@ -126,7 +150,13 @@ pub(crate) fn plugin_thread_main(
                     client_id,
                 )));
             },
-            PluginInstruction::Exit => break,
+            PluginInstruction::ApplyCachedEvents(plugin_id) => {
+                wasm_bridge.apply_cached_events(plugin_id)?;
+            },
+            PluginInstruction::Exit => {
+                wasm_bridge.cleanup();
+                break;
+            },
         }
     }
     info!("wasm main thread exits");
