@@ -639,6 +639,7 @@ mod config_test {
             ui {
                 pane_frames {
                     rounded_corners true
+                    hide_session_name true
                 }
             }
         "#;
@@ -646,6 +647,7 @@ mod config_test {
         let expected_ui_config = UiConfig {
             pane_frames: FrameConfig {
                 rounded_corners: true,
+                hide_session_name: true,
             },
         };
         assert_eq!(config.ui, expected_ui_config, "Ui config defined in config");
