@@ -394,6 +394,7 @@ pub struct TiledPaneLayout {
     pub focus: Option<bool>,
     pub external_children_index: Option<usize>,
     pub children_are_stacked: bool,
+    pub is_expanded_in_stack: bool,
 }
 
 impl TiledPaneLayout {
@@ -569,6 +570,13 @@ impl TiledPaneLayout {
         // if max_panes is 1, it means there's only enough panes for this node,
         // if max_panes is 0, this is probably the root layout being called with 0 max panes
         if max_panes <= 1 {
+            while !self.children.is_empty() {
+                // this is a special case: we're truncating a pane that was previously a logical
+                // container but now should be an actual pane - so here we'd like to use its
+                // deepest "non-logical" child in order to get all its attributes (eg. borderless)
+                let first_child = self.children.remove(0);
+                drop(std::mem::replace(self, first_child));
+            }
             self.children.clear();
         } else if max_panes <= self.children.len() {
             self.children.truncate(max_panes);
@@ -858,12 +866,15 @@ fn split_space(
 ) -> Result<Vec<(TiledPaneLayout, PaneGeom)>, &'static str> {
     let mut pane_positions = Vec::new();
     let sizes: Vec<Option<SplitSize>> = if layout.children_are_stacked {
+        let index_of_expanded_pane = layout.children.iter().position(|p| p.is_expanded_in_stack);
         let mut sizes: Vec<Option<SplitSize>> = layout
             .children
             .iter()
             .map(|_part| Some(SplitSize::Fixed(1)))
             .collect();
-        if let Some(last_size) = sizes.last_mut() {
+        if let Some(index_of_expanded_pane) = index_of_expanded_pane {
+            *sizes.get_mut(index_of_expanded_pane).unwrap() = None;
+        } else if let Some(last_size) = sizes.last_mut() {
             *last_size = None;
         }
         sizes
