@@ -1881,6 +1881,21 @@ fn can_define_stacked_children_for_pane_template() {
     assert_snapshot!(format!("{:#?}", layout));
 }
 
+#[test]
+fn can_define_a_stack_with_an_expanded_pane() {
+    let kdl_layout = r#"
+        layout {
+           pane stacked=true {
+               pane
+               pane expanded=true
+               pane
+           }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None, None).unwrap();
+    assert_snapshot!(format!("{:#?}", layout));
+}
+
 #[test]
 fn cannot_define_stacked_panes_for_bare_node() {
     let kdl_layout = r#"
@@ -1892,6 +1907,20 @@ fn cannot_define_stacked_panes_for_bare_node() {
     assert!(layout.is_err(), "error provided for tab name with space");
 }
 
+#[test]
+fn cannot_define_an_expanded_pane_outside_of_a_stack() {
+    let kdl_layout = r#"
+        layout {
+            pane {
+                pane
+                pane expanded=true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None, None);
+    assert!(layout.is_err(), "error provided for tab name with space");
+}
+
 #[test]
 fn cannot_define_stacked_panes_with_vertical_split_direction() {
     let kdl_layout = r#"
