@@ -2,7 +2,9 @@ pub mod cli;
 pub mod consts;
 pub mod data;
 pub mod envs;
+pub mod errors;
 pub mod input;
+pub mod kdl;
 pub mod pane_size;
 pub mod position;
 pub mod setup;
@@ -12,14 +14,12 @@ pub mod shared;
 #[cfg(not(target_family = "wasm"))]
 pub mod channels; // Requires async_std
 #[cfg(not(target_family = "wasm"))]
-pub mod errors; // Requires async_std (via channels)
-#[cfg(not(target_family = "wasm"))]
 pub mod ipc; // Requires interprocess
 #[cfg(not(target_family = "wasm"))]
 pub mod logging; // Requires log4rs
 
 #[cfg(not(target_family = "wasm"))]
 pub use ::{
-    anyhow, async_std, clap, interprocess, lazy_static, libc, nix, regex, serde, serde_yaml,
-    signal_hook, tempfile, termwiz, vte,
+    anyhow, async_std, clap, interprocess, lazy_static, libc, nix, regex, serde, signal_hook,
+    tempfile, termwiz, vte,
 };
